﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Globalization;
using System.IO.Ports;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace Delta_Test
{
    /// <summary>
    /// Класс для работы с преобразователями Delta серии CP2000 по протоколу MODBUS ASCII
    /// </summary>
	public class DeltaModBusASCII
	{
        /// <summary>
        /// Сылка на объект соотв. компорта, через который отсылаются RS-485 MODBUS сообщения
        /// </summary>
        public SerialPort Port;

		/// <summary>
		/// Коммуникационный адрес ПЧ - значения из диапазона 0...254. Адрес со значением '0' означает, что данные будут переданы всем устройствам,
		/// причем ответное сообщение не формируется при этом. По умолчанию значение адреса равно '1'.
		/// </summary>
		public byte CommunicationAddress
		{
			get
			{
				return communicationAddress;
			}
			set
			{
				if (value <= 254)
				{
					communicationAddress = value;
				}
			}
		}
		private byte communicationAddress = 0x01;

		/// <summary>
		/// Флаг того, что пришел строковый ответ от преобразователя
		/// </summary>
		public bool IsStringReceived = false;

        /// <summary>
        /// Принятая строка от драйвера. Если еще ничего не принято в ответ, то возвращает нулевое значение.
        /// </summary>
        public string ReceivedString
        {
            get
            {
                if (IsStringReceived == true)
                {
                    IsStringReceived = false;
                    return receivedString;
                }
                else
                {
                    return null;
                }
            }
        }
        private string receivedString = ":", receivedData = "";

        /// <summary>
		/// Адреса параметров драйвера CP2000 и их символические обозначения
		/// </summary>
		public enum Parameter : ushort
		{
			#region 00 Drive Parameters

			/// <summary>
			/// Идентификационный код драйвера. Только чтение. (пример: "23" это 22kW, 460V)
            /// </summary>
            p0000_DriveID = 0x0000,

            /// <summary>
            /// Номинальный ток драйвера. Только чтение.
            /// </summary>
            p0001_RatedCurrent = 0x0001,

            /// <summary>
            /// Сброс все параметров. Записываемый параметр. Значение 9 - дает сброс всех параметров для 50Гц сети.
            /// </summary>
            p0002_ParameterReset = 0x0002,

            /// <summary>
            /// Версия ПО. Только чтение.
            /// </summary>
            p0006_SoftwareVersion = 0x0006,

            /// <summary>
			/// Режим управления скоростью. Записываемый параметр. 0 - скалярное управление (VF); 2 - бездатчиковое векторное управление (SVC)
            /// </summary>
            p0011_VelocityControlMode = 0x0011,

            /// <summary>
			/// Источник задания частоты привода в режиме AUTO. Записываемый параметр. 0 - Digital keypad, 1 - RS-485 communication,
			/// 2 - External analog input (Pr.03-00), 3 - External UP/DOWN terminal, 6 - CANopen communication card, 8 - Communication card (no CANopen card)
            /// </summary>
            p0020_SourceOfAUTOforFrequency = 0x0020,
 
            /// <summary>
			/// Источник задания команд для привода в режиме AUTO. Записываемый параметр. 0 - Digital keypad, 1 - External analog input (Pr.03-00),
			/// 2 - RS-485 communication, 3 - External UP/DOWN terminal, 5 - Communication card (no CANopen card)
            /// </summary>
            p0021_SourceOfAUTOforOperation = 0x0021,

            /// <summary>
			/// Способ останова двигателя. Записываемый параметр. 0 - с заданным замедлением, 1 - на свободном выбеге.
            /// </summary>
            p0022_StopMethod = 0x0022,

            /// <summary>
			/// Управление направлением вращения двигателя. Записываемый параметр. 0 - Enable forward/reverse; 1 - Reverse disable; 2 - Forward disable.
            /// </summary>
            p0023_MotorDirectionControl = 0x0023,

			/// <summary>
			/// Пользовательские настройки для отображения характеристик. Записываемый параметр.
			/// Биты 0...3 - задание кол-ва знаков после запятой :
			/// 0 - целое число; 1 - 1 знак после запятой; 2 - 2 знака после запятой; 3 - 3 знака после запятой;
			/// Биты 4...15 - единица измерения, для которой устанавливается/читается соотв. значение битов 0...3 :
			/// (Внимание! Поодерживаются только 4 первых из нижеуказанных единиц измерения)
			/// 000xH: Hz
			/// 001xH: rpm
			/// 002xH: %
			/// 003xH: kg
			/// 004xH: m/s
			/// 005xH: kW
			/// 006xH: HP
			/// 007xH: ppm
			/// 008xH: 1/m
			/// 009xH: kg/s
			/// 00AxH: kg/m
			/// 00BxH: kg/h
			/// 00CxH: lb/s
			/// 00DxH: lb/m
			/// 00ExH: lb/h
			/// 00FxH: ft/s
			/// 010xH: ft/m
			/// 011xH: m
			/// 012xH: ft
			/// 013xH: degC
			/// 014xH: degF
			/// 015xH: mbar
			/// 016xH: bar
			/// 017xH: Pa
			/// 018xH: kPa
			/// 019xH: mWG
			/// 01AxH: inWG
			/// 01BxH: ftWG
			/// 01CxH: psi
			/// 01DxH: atm
			/// 01ExH: L/s
			/// 01FxH: L/m
			/// 020xH: L/h
			/// 021xH:m3/s
			/// 022xH: m3/h
			/// 023xH: GPM
			/// 024xH:CFM 
			/// </summary>
			p0025_UserDefinedProperty = 0x0025,

			/// <summary>
			/// Переключение из режима AUTO в режим HAND. Записываемый параметр. 0 - Enable forward/reverse; 1 - Reverse disable; 2 - Forward disable.
			/// </summary>
			p0028_SwitchingFromAutoHandMode = 0x0028,

			/// <summary>
			/// Выбор для локального или удаленного управления. Записываемый параметр. 0 - Enable forward/reverse; 1 - Reverse disable; 2 - Forward disable.
			/// </summary>
			p0029_LocalRemoteSelection = 0x0029,











			#endregion

			#region 01 Basic Parameter



			#endregion

			#region 02 Digital Input/Output Parameter



			#endregion

			#region 03 Analog Input/Output Parameter



			#endregion

			#region 04 Multi-Step Speed Parameters



			#endregion

			#region 05 Motor Parameters

			/* 
05 - 26 
Accumulative Watt Per Second of Motor in Low Word (W-sec) 
           Factory Setting: 0.0 
  Settings Read only 
 
05 - 27 
Accumulative Watt Per Second of Motor in High Word (W-sec) */


			/// <summary>
			/// Этот параметр используется для записи значения времени работы двигателя в минутах. При записи 0x00 соотв. время работы сбрасывается сбрасывается.
			/// </summary>
			p0531_MotorOperationTime_Min = 0x0531,

			/// <summary>
			/// Этот параметр используется для записи значения времени работы двигателя в днях. При записи 0x00 соотв. время работы сбрасывается сбрасывается.
			/// </summary>
			p0532_MotorOperationTime_Days = 0x0532,

			#endregion

			#region 06 Protection Parameters

			#endregion

			#region 07 Special Parameters



			#endregion

			#region 08 High-function PID Parameters



			#endregion

			#region 09 Communication Parameters



			#endregion

			#region 12 Pump Parameter



			#endregion

			#region 20 Command (Write only)

			/// <summary>
			/// Команда драйверу (только для записи) - некоторые доступные значения можно взять из enum Command
			/// </summary>
			p2000_COMMAND = 0x2000,

			#endregion

			#region 21 22 Status monitor (Read only)

			/* Оставшиеся не включенные параметры:
				* 2102H Frequency command (F)
				* 2105H DC-BUS Voltage (UXXX.X)
				* 2107H Current step number of Multi-Step Speed Operation
				* 2116H Multi-function display (Pr.00-04)
				* 211BH Max. setting frequency
				* 2201H Display counter value of TRG terminal (c)
				* 2202H Display actual output frequency (H)
				* 2203H Display DC-BUS voltage (u)
				*/

			/// <summary>
			/// Код ошибки (чтение) - см. параметры 06-17...06-22
			/// </summary>
			p2100_ERRORCODE = 0x2100,

			/// <summary>
			/// Состояние ПЧ (чтение) - для расшифрования состояния использовать структуру типа DRIVER_STATUS
			/// </summary>
			p2101_DRIVESTATUS = 0x2101,

			/// <summary>
			/// Выходная частота, Гц (H)
			/// </summary>
			p2103_OUTPUT_FREQUENCY = 0x2103,

			/// <summary>
			/// Выходной фазный ток, А (AXXX.X)
			/// </summary>
			p2104_OUTPUT_CURRENT = 0x2104,

			/// <summary>
			/// Выходное межфазное напряжение, В (EXXX.X)
			/// </summary>
			p2106_OUTPUT_VOLTAGE = 0x2106,

			/// <summary>
			/// Счетчик моточасов
			/// </summary>
			p2109_COUNTER_VALUE = 0x2109,

			/// <summary>
			/// Коэффициент мощности (XXX.X)
			/// </summary>
			p210A_POWER_FACTOR_ANGLE = 0x210A,

			/// <summary>
			/// Выходной момент в %
			/// </summary>
			p210B_OUTPUT_TORQUE = 0x210B,

			/// <summary>
			/// Частота вращения двигателя в данный момент времени (об/мин)
			/// </summary>
			p210C_ACTUALMOTORSPEED = 0x210C,

			/// <summary>
			/// Выходная мощность в kW (X.XXX)
			/// </summary>
			p210F_POWER_OUTPUT = 0x210F,

			/// <summary>
			/// Display output current (A)
			/// </summary>
			p2200_DISPLAY_OUTPUT_CURRENT = 0x2200,

			/// <summary>
			/// Температура IGBT модуля в °C
			/// </summary>
			p220E_TEMPERATURE_IGBT = 0x220E,

			/// <summary>
			/// Температура внутри преобразователя (около конденсаторов) в °C
			/// </summary>
			p220F_TEMPERATURE_INSIDE = 0x220F

			#endregion
		}

		/// <summary>
		/// Некоторые из значений для записи в параметр COMMAND
		/// </summary>
		public enum COMMAND
		{
			/// <summary>
			/// Биты 0...3. Значение - "Нет функции". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits0_3_NO_FUNCTION = 0,
			/// <summary>
			/// Биты 0...3. Значение - "Стоп двигателю". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits0_3_STOP = 1,
			/// <summary>
			/// Биты 0...3. Значение - "Запуск двигателя". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits0_3_RUN = 2,
			/// <summary>
			/// Биты 0...3. Значение - "Команда JOG и работа". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits0_3_JOG_PLUS_RUN = 3,
			/// <summary>
			/// Биты 4...5. Значение - "Нет функции". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits4_5_NO_FUNCTION = 0x00,
			/// <summary>
			/// Биты 4...5. Значение - "Прямое вращение". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits4_5_FWD = 0x10,
			/// <summary>
			/// Биты 4...5. Значение - "Обратное вращение". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits4_5_REV = 0x20,
			/// <summary>
			/// Биты 4...5. Значение - "Смена направления вращения". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits4_5_CHANGE_DIRECTION = 0x30,
			/// <summary>
			/// Биты 13...14. Значение - "Нет функции". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits13_14_NO_FUNCTION = 0x0000,
			/// <summary>
			/// Биты 13...14. Значение - "Источник команд - пульт". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits13_14_OPERATED_BY_KEYPAD = 0x2000,
			/// <summary>
			/// Биты 13...14. Значение - "Источник команд как в параметре 00-21". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits13_14_OPERATED_BY_Pr0021_SETTING = 0x4000,
			/// <summary>
			/// Биты 13...14. Значение - "Смена источника команд". Выберите только одно значение этих бит в команде и складывайте со значениями  др. бит.
			/// </summary>
			Bits13_14__CHANGE_OPERATION_SOURCE = 0x6000
		}

		/// <summary>
		/// Флаги состояний преобразователя (эти флаги только внутренние, а устанавливает их
		/// функция GetDriverStatusFromParametrData в соответствии с реальным состоянием преобразователя)
		/// </summary>
		[Flags]
		public enum STATUS
		{
			/// <summary>
			/// Преобразователь остановлен
			/// </summary>
			Stops = 1,
			/// <summary>
			/// Преобразователь замедляет движение
			/// </summary>
			Decelerating = 2,
			/// <summary>
			/// Преобразователь в дежурном режиме
			/// </summary>
			Standby = 4,
			/// <summary>
			/// Преобразователь в рабочем режиме
			/// </summary>
			Operating = 8,
			/// <summary>
			/// Преобразователь исполняет команду JOG
			/// </summary>
			JogCommand = 16,
			/// <summary>
			/// Преобразователь работает в прямом направлении
			/// </summary>
			FWDrun = 32,
			/// <summary>
			/// Преобразователь переходит с обратного направления на прямое
			/// </summary>
			FromREVtoFWDrun = 64,
			/// <summary>
			/// Преобразователь работает в обратном направлении
			/// </summary>
			REVrun = 128,
			/// <summary>
			/// Преобразователь переходит с прямого направления на обратное
			/// </summary>
			FromFWDtoREVrun = 256,
			/// <summary>
			/// Частота преобразователя задается через коммуникационный интерфейс
			/// </summary>
			FreqContrByCommInter = 512,
			/// <summary>
			/// Частота преобразователя задается через аналоговый вход
			/// </summary>
			FreqContrByAnalogSignal = 1024,
			/// <summary>
			/// Команды преобразователя задаются через коммуникационный интерфейс
			/// </summary>
			OperContrByCommInter = 2048,
			/// <summary>
			/// Параметры преобразователя заблокированы
			/// </summary>
			ParametersLocked = 4096,
			/// <summary>
			/// Разрешено копирование параметров из пульта управления
			/// </summary>
			EnableToCopyParameterFromKeypad = 8192
		}

		/// <summary>
		/// Командные коды преобразователю. В MODBUS ASCII для ПЧ Delta - это 2 байта(цифры в символьном отображении) после адреса ПЧ
		/// </summary>
		public enum Function
		{
			/// <summary>
			/// Чтение данных из регистров
			/// </summary>
			READ_REGISTERS = 0x03,
			/// <summary>
			/// Запись данных в один регистр
			/// </summary>
			WRITE_REGISTER = 0x06,
			/// <summary>
			/// Детектирование цикла
			/// </summary>
			CYCLE_DETECTED = 0x08,
			/// <summary>
			/// Запись данных в несколько регистров
			/// </summary>
			WRITE_REGISTERS = 0x10
		}

		/// <summary>
        /// Конструктор класса для работы с преобразователями Delta серии CP2000.
        /// </summary>
        /// <param name="port">Объект компорта, через который будет работать протокол MODBUS ASCII</param>
        /// <param name="portName">Имя компорта</param>
        /// <param name="baudRate">Скорость передачи. По умолчанию 9600 в преобразователях Delta серии CP2000</param>
        /// <param name="dataBits">Число бит в данных. По умолчанию 7 в преобразователях Delta серии CP2000</param>
        /// <param name="parity">Контроль четности. По умолчанию нет(None) в преобразователях Delta серии CP2000</param>
        /// <param name="stopBits">Число стоповых бит. По умолчанию 2 в преобразователях Delta серии CP2000</param>
		public DeltaModBusASCII(string portName, int baudRate, Parity parity, int dataBits, StopBits stopBits)
        {
			Port = new SerialPort(portName, baudRate, parity, dataBits, stopBits);
            Port.Encoding = Encoding.ASCII;
            try
            {
                Port.Open();
                Port.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(this.Port_DataReceived);
            }
            catch
            {
                throw new ApplicationException("Невозможно открыть данный компорт! Возможно он занят другим приложением.");
            }
        }

		public DeltaModBusASCII()
		{
			
		}

		private void Port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // принимаем строковые данные
            receivedData = receivedData + Port.ReadExisting();
			// если в конце принятых строковых данных есть символ LF(/n или 0x0a), то сохраняем принятое в соотв. строке receivedString
            if (receivedData.EndsWith("\n"))
            {
				receivedString = receivedData.Substring(0);
                receivedData = "";
                IsStringReceived = true;
            }
        }

		/// <summary>
		/// Отправка MODBUS ASCII сообщения преобразователю
		/// </summary>
		/// <param name="function"> Командный код сообщения (запись в регистры, чтение и пр.) </param>
		/// <param name="parametr"> Адрес регистра (параметра) </param>
		/// <param name="parametrValue"> Кол-во слов для чтения (при чтении регистра(-ов)) или данные для записи в параметр (при записи в регистр) </param>
		/// <returns></returns>
		public bool SendMessage(Function function, Parameter parametr, int parametrValue)
        {
            try
            {
				if (this.Port.IsOpen)
				{
					string sendStr = ":" + communicationAddress.ToString("X2") + ((int)function).ToString("X2") + ((int)parametr).ToString("X4") + parametrValue.ToString("X4") + LRC_Calculate(function, parametr, parametrValue).ToString("X2") + "\r\n";

					Port.WriteLine(sendStr);
					
					return true;
				}
				else
				{
					return false;
				}
            }
            catch
            {
                return false;
            }
        }

		public bool SendMessage(Parameter parametr, int[] parametrValues, int countOfValues)
		{
			try
			{
				return true;
			}
			catch
			{
				return false;
			}
		}

		/// <summary>
		/// Подсчет LRC (Longitudinal Redundancy Check) для записываемых данных одного регистра (параметра).
		/// LRC рассчитывается след. образом - суммируются значения байтов начиная с коммуникационного адреса
		/// до последнего символа данных и результат вычитается из 0x100
		/// </summary>
		/// <param name="function"> Командная функция преобразователю. </param>
		/// <param name="parametr"> Адрес параметра(регистра). </param>
		/// <param name="parametrValue"> Записываемое значение в регистр. </param>
		/// <returns></returns>
		public byte LRC_Calculate(Function function, Parameter parametr, int parametrValue)
		{
			int value0x100 = 0x100;
			byte sum = communicationAddress;

			sum += (byte)((int)function);
			sum += (byte)((int)parametr >> 8);
			sum += (byte)parametr;
			sum += (byte)((int)parametrValue >> 8);
			sum += (byte)parametrValue;
			return (byte)(value0x100 - sum);
		}

		/// <summary>
		/// Извлечение данных параметров из строкового ответа на запрос чтения этих параметров
		/// </summary>
		/// <param name="response"> строка-ответ полностью (например в таком виде: ":010620000002D7\r\n" или  ":0103022100D9\r\n" и т.п.) </param>
		/// <param name="parametersData"> массив с параметрами, заполняетсяЫ в этой функции </param>
		/// <param name="countOfBytes"> число байт с данными в принятой строке, также заполняет эта функция </param>
		/// <returns></returns>
		public bool GetParametersDataFromResponse(string response, out ushort[] parametersData, out ushort countOfBytes)
		{
			int data;

			parametersData = null;
			countOfBytes = 0;
			// вначале проверяем, что данный строковый ответ от преобразователя соотв. команде чтения параметров
			if (Int32.TryParse(response.Substring(3, 2), System.Globalization.NumberStyles.HexNumber, CultureInfo.InvariantCulture, out data) == true && data == (int)Function.READ_REGISTERS)
			{
				// вычисляем кол-во байт в ответе - если возможно
				if (UInt16.TryParse(response.Substring(5, 2), System.Globalization.NumberStyles.HexNumber, CultureInfo.InvariantCulture, out countOfBytes) == true)
				{
					// т.к. у нас для считываемых данных принят массив UInt16 чисел, то определяем требуемую длину массива след. образом
					parametersData = new ushort[countOfBytes / 2 + (((countOfBytes % 2) == 0) ? 0 : 1)];
					// теперь считываем последовательно, словами, все полученные данные
					for (int i = 0; i < parametersData.Length; i++)
					{
						// если вдруг невозможно будет считать какой-то числовой параметр, то выход с ошибкой (тут 4 т.к. 4 символа дают одно UInt16 число)
						if (UInt16.TryParse(response.Substring(7 + 4 * i, 4), System.Globalization.NumberStyles.HexNumber, CultureInfo.InvariantCulture, out parametersData[i]) != true)
						{
							return false;
						}
					}
					// тут выходим с успешным завершением функции
					return true;
				}
				// иначе - т.е. невозможно определить кол-во байт в ответе
				else
				{
					return false;
				}
			}
			// иначе - т.е. это другой ответ или не явл. правильным ответом на считываемые параметры
			else
			{
				return false;
			}
		}

		/// <summary>
		/// Чтение значения только одного параметра из преобразователя
		/// </summary>
		/// <param name="parametr"> Адрес параметра, который нужно прочитать </param>
		/// <param name="parametrValue"> Возвращаемое прочтенное значение этого параметра </param>
		/// <returns> Возвращает 'true' в случае успешного чтения </returns>
		public bool ReadOneParameterValue(Parameter parametr, out ushort parametrValue)
		{
			ushort countOfBytes = 0, passCounter = 0;
			ushort[] parametrValues;

			// отправляем сообщение, что хочем прочитать одно значение заданного параметра
			SendMessage(Function.READ_REGISTERS, parametr, 1);

			// ожидаем ответ от преобразователя,  при этом подсчитывая время в мс и при достижении лимита - выход с ошибкой
			while (IsStringReceived != true)
			{
				// пауза в 1 мс
				Thread.Sleep(1);
				// не достигнут ли лимит - при 9600 бит/сек отправка и прием ~ максимум по 50 мс каждые
				if (++passCounter > 150)
				{
					// выход с ошибкой
					parametrValue = 0;
					return false;
				}
			}

			// если уже принят ответ от преобразователя и удалось получить принятые данные в виде числа, то соотв. успешный выход
			if (GetParametersDataFromResponse(ReceivedString, out parametrValues, out countOfBytes) == true)
			{
				parametrValue = parametrValues[0];
				return true;
			}
			// иначе - выход с ошибкой
			else
			{
				parametrValue = 0;
				return false;
			}
		}

		/// <summary>
		/// Определение состояния ПЧ в зависимости от считанного значения параметра 21-01
		/// </summary>
		/// <param name="parametr2101_Data"> считанное значение параметра 21-01 </param>
		/// <returns></returns>
		public STATUS GetDriverStatusFromParametrData(ushort parametr2101_Data)
		{
			STATUS status;

			// вначале анализируем 2 младшие биты значения параметра 21-01 и по результату устанавливаем соотв. флаг
			switch (parametr2101_Data & 0x03)
			{
				// если 2 первых бита равны - 0b01
				case 1:
					status = STATUS.Decelerating;
					break;
				// если 2 первых бита равны - 0b10
				case 2:
					status = STATUS.Standby;
					break;
				// если 2 первых бита равны - 0b11
				case 3:
					status = STATUS.Operating;
					break;
				// если 2 первых бита равны - 0b00
				default:
					status = STATUS.Stops;
					break;
			}

			// анализируем 2-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 2)) == (1 << 2))
			{
				status |= STATUS.JogCommand;
			}

			// анализируем биты 3 и 4 значения параметра 21-01 и по результату устанавливаем соотв. флаг
			switch ((parametr2101_Data >> 3) & 0x03)
			{
				// если биты 3 и 4 равны - 0b01
				case 1:
					status |= STATUS.FromREVtoFWDrun;
					break;
				// если биты 3 и 4 равны - 0b10
				case 2:
					status |= STATUS.FromFWDtoREVrun;
					break;
				// если биты 3 и 4 равны - 0b11
				case 3:
					status |= STATUS.REVrun;
					break;
				// если биты 3 и 4 равны - 0b00
				default:
					status |= STATUS.FWDrun;
					break;
			}

			// анализируем 8-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 8)) == (1 << 8))
			{
				status |= STATUS.FreqContrByCommInter;
			}

			// анализируем 9-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 9)) == (1 << 9))
			{
				status |= STATUS.FreqContrByAnalogSignal;
			}

			// анализируем 10-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 10)) == (1 << 10))
			{
				status |= STATUS.OperContrByCommInter;
			}

			// анализируем 11-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 11)) == (1 << 11))
			{
				status |= STATUS.ParametersLocked;
			}

			// анализируем 12-й бит в считанном ответе
			if ((parametr2101_Data & (1 << 12)) == (1 << 12))
			{
				status |= STATUS.EnableToCopyParameterFromKeypad;
			}

			// возвращаем соотв. установленным значение статуса преобразователя
			return status;
		}
	}
}

//public void ReceiveCharEvent(object s, EventArgs e)
//{
//byte[] data = Encoding.Default.GetBytes(sendStr);
//}
